from distutils.core import setup

# NOTE Remember to change version in rattler.py as well.
setup(name="rattler", version="1.0.2",
      author="Yellow Agents AB",
      author_email="ludvig@yellowagents.com",
      url="http://yellowagents.com/seismometer/",
      py_modules=["rattler"])
